# signature { license=0, seed=457995617, key="50da3a208a7c68385c1ea76e221be4ba" }
--






local cgi = require( 'cgi' )
local lfs = require( 'lfs' )


local runningApps={}


local maxUploadSize = 1024*1024




MsgBox = ''

local lang = cgi.ACCEPT_LANGUAGE
lang = lang and lang:match( '^%s*(%a%a)' )













local cwd = package.path:gsub( '([?].*)', '' ):sub(1,-6)


local Z = package.config:sub(1,1)







local function map( ... )
	local f = table.pack( cwd, ... )
	return (table.concat( f, Z ):gsub( '/', Z ))
end

local attr = lfs.attributes




local function exists( ... ) return attr( map( ... ), 'size' ) end



local function isdir( ... ) 
	local dir = map( ... )
	return attr( dir, 'mode' ) == 'directory'
end



local lfsdir


local lower

if Z == '/' then

	lfsdir = function( pattern )
		local path, mask = pattern:match( '^(.-/)([^/]+)$' )

		mask = mask and '^'..mask:gsub('[.]','%%.'):gsub('*','.-')..'$' or '.-'

		local dir, context = lfs.dir( path )
		local f = function( )
			while true do
				local name = dir( context )
				if not name then return nil end
				if name:find( mask ) then return name end
			end
		end
		return f
	end

	lower = function(s) return s end
else

	lfsdir = lfs.dir

	lower = string.lower
end


local function unescape(s)
	s = string.gsub( s, '+', ' ' )
	return string.gsub( s, "%%(%x%x)",
		function( hex )
			return string.char( tonumber( hex, 16 ) )
		end
	)
end



local i18nTexts = {}

do
	local code = loadfile( map( 'html/i18n.'..lang ), 't', i18nTexts )
	          or loadfile( map( 'html/i18n.txt' ), 't', i18nTexts )
	if code then
		code()
		i18nTexts = i18nTexts.i18n
	end
end



function i18n( key, default ) return i18nTexts[ key ] or default or key end




local PermissionTexts =
{
	OS_MEMORY   = i18n("use up to %d bytes of main memory");
	OS_KILL     = i18n("stop other applications");
	LFS_MKDIR   = i18n("create directories on disk");
	LFS_RMDIR   = i18n("delete directories on disk");
	LFS_TOUCH   = i18n("change a file's time stamp");
	LFS_SETATTR = i18n("change a file's attributes");
	INST_LIB    = i18n("install a library module");
	UPD_APPMGR  = i18n("update the application manager");
}


setmetatable( PermissionTexts, { __index = function( _, p ) return string.format( i18n("use system operation '%s'"), p ) end } )





local function rm_rf( path )
	if attr( path, 'mode' ) ~= 'directory' then return end

	require 'permissions'
	for file in lfsdir( path..Z..'*' ) do
		if file ~= '.' and file ~= '..' then
			local f = path..Z..file
			if attr( f, 'mode' ) == 'directory' then
				rm_rf( f )
			else
				os.remove( f )
			end
		end
	end

	lfs.rmdir( path )
end



local function saveTo( data, path )
	os.remove( path )
	local f = io.open( path, 'w' )
	if f then
		f:write( data )
		f:close()
		return true
	end
end





local function findRunningApps()
	runningApps = {}
	for i=1,1000 do
		local state,path = os.psinfo(i)

		if not state then break end
		path = string.lower(path or "")

		if state > 0 and state < 4 then
			local name= path:match( '[/\\]([^/\\]+)[/\\]%1[-.]lua$' )
			if name then
				runningApps[ name ] = i
				runningApps[ i ] = name
			end
		end
	end
end





local function preInstall( posted )
	require 'permissions'
	local zip = require( 'zip' )

	local tmp = map( 'html/manager.tmp' )
	local mnf = map( 'html/manifest.tmp' )
	saveTo( posted, tmp )
	local zipF = zip.open( tmp )
	if not zipF then return nil, i18n("package has wrong file format!") end

	local m = zipF:extract( 'manifest' )
	if not m or not m.data       then return nil, i18n("package contains no manifest") end
	if not saveTo( m.data, mnf ) then return nil, i18n("can't write manifest") end
	if os.license( mnf ) == nil  then return nil, i18n("manifest has an invalid signature") end


	local manifest = {}
	local code = loadfile( mnf, 't', manifest )
	if not code then return nil, i18n("manifest is not a valid Lua script") end
	code()
	local name = manifest.name
	if not name then return nil, i18n("manifest contains no valid name") end


	lfs.mkdir( map( name ) )

	if isdir( name ) then

		if not exists( name, 'html/icon.png' ) then
			local icon = zipF:extract( 'html/icon.png' )
			if icon and icon.data then
				lfs.mkdir( map( name, 'html' ) )
				saveTo( icon.data, map( name, 'html/icon.png' ) )
			end
		end
	end
	zipF:close()


	os.rename( tmp, map( name, name..'.pkg' ) )
	os.rename( mnf, map( name, 'manifest.pkg' ) )

	return name
end




local function unInstall( name )
	if name == 'lib' or name == 'html' then return end -- these are protected

	if exists( name, 'uninstall.lua') then
		local uninstaller = map( name, 'uninstall.lua' )
		if os.license( uninstaller ) then pcall( dofile, uninstaller ) end
	end
	rm_rf( map( name ) )
end 



local function dontInstall( name )
	require 'permissions'


	if name ~= 'lib' and name ~= 'html' 
	and not exists( name, 'manifest' ) then
		os.remove( map( name, 'html/icon.png' ) )
		lfs.rmdir( map( name, 'html' ) )
	end


	os.remove( map( name, 'manifest.pkg' ) )
	os.remove( map( name, name..'.pkg' ) )
end



local function doInstall( name )
	require 'permissions'

	local zip = require( 'zip' )


	local pkg = map( name, name..'.pkg' )
	local zipF, err = zip.open( pkg )
	if not zipF then
		MsgBox = i18n("zip error")..':\\n'..err
		return
	end



	local function unpackFiles( path, T, target )

		for name,subT in pairs( T ) do
			if type( subT ) == 'table' then
				lfs.mkdir( target..name )
				unpackFiles( path..name..'/', subT, target..name..Z)
			end
		end

		for _,name in ipairs( T ) do
			local fname = path..name
			local m = zipF:extract( fname )
			if m and m.data then saveTo( m.data, target..name ) end
		end
	end



	local mfname = map( name, 'manifest.pkg' )
	local manifest = {}
	local code = loadfile( mfname, 't', manifest )
	if code then
		code()
		local T = manifest.files
		unpackFiles( '', T, map( name, '' ) )
	end

	zipF:close()

	os.remove( pkg )
	os.remove( mfname )


	if exists( name, name..'-lua') then

		os.remove( map( name, name..'-lua' ) )
		os.rename( map( name, name..'.lua' ), map( name, name..'-lua' ) )
	end


	if exists( name, 'install.lua') then
		local installer = map( name, 'install.lua' )
		if os.license( installer ) then pcall( dofile, installer ) end
	end
end




local function getManifest( p )
	local M = {}
	local code = loadfile( p, 't', M )
	if code then
		code()
		return M
	end
end








local function getAppDetails( name )
	if name=='.' or name=='..' or not isdir( name ) then return end

	local app = {
		name   = name ,
		icon   = exists( name, 'html/icon.png' ) ,
		config = exists( name, 'html/config.cgi' ) ,
		M      = getManifest( map( name, 'manifest' ) ) ,
		N      = getManifest( map( name, 'manifest.pkg' ) )
	}
	app.license = ( app.N or app.M or {} ).license

	if exists( name, name..'.lua' ) then
		app.canRun = true
	elseif exists( name, name..'-lua' ) then
		app.isLocked = true
		app.canRun = true
	end

	if exists( name, 'html', name..'.cgi' ) then app.home = name..'.cgi' end
	if runningApps[ name ] then app.canStop = true	end

	if app.N then return app end
	if not app.M then return end
	if name == 'lib' or name == 'html' then return end

	return app
end





local function paragraph( title, formatstring, ... )
	local t = select( 1, ... )
	if not t or t == '' then return '' end
	t = string.format( formatstring, ...)
	cgi.printf( '<h5>%s</h5>%s<br />', title, t or '')
end





local function listPermissions( P )
	local ph = false;
	for k,v in pairs( P ) do
		if not ph then
			cgi.print( '<h5>', i18n("Required Permissons"), '</h5><br />', i18n("This application requires your permission to"), '<ul>')
			ph = true
		end
		cgi.print( '<li>' )
		cgi.printf( PermissionTexts[ k ], v )
		cgi.print( '</li/>' )
	end
	if ph then cgi.print( '</ul>' ) end
end




local function listRequirements( R )
	local function get( name )
		local loaded = package.loaded
		package.loaded = {}
		local ok, L = pcall( require, name )
		package.loaded = loaded
		if ok then return L end
	end


	if R.licenses then

		local ph = false

		for lic, needed in pairs( R.licenses ) do
			local installed = os.license( tonumber( lic ) ) 
			needed = tonumber( needed ) or 1
			if installed < needed then
				if not ph then
					cgi.print('<h5>', i18n("Required Licenses"), '</h5><br />', i18n("This application requires to install"),"<ul>")
					ph = true;
				end
				cgi.printf( '<li>%d %s %d</li/>', needed - installed, i18n("instance(s) of License"), lic )
			end
		end

		if ph then cgi.print( '</ul>' ) end
	end


	if R.libraries then
		local ph = false
		for library, version in pairs( R.libraries ) do
			local versionOK
			local V = version

			local lib = get( library )
			if not lib then

				versionOK = false
				V = ''
			else

				local imajor, iminor = ( lib._VERSION or 'version 0.0' ):match( '^%w+%s+(%d+)[.]?(%d*)' )
				versionOK = type( version ) ~= 'string'
				imajor = tonumber( imajor ) or 0
				iminor = tonumber( iminor ) or 0
				if versionOK then

					V = '0.0'
				else


					local wmajor,wminor = version:match( '(%d+)[.]?(%d*)' )
					wmajor = tonumber( wmajor ) or 0
					wminor = tonumber( wminor ) or 0

					versionOK = (imajor > wmajor) or (imajor == wmajor and iminor >= wminor) 
				end
			end

			if not versionOK then

				if not ph then
					cgi.print( '<h5>', i18n("Required Libraries"), '</h5><br />', i18n("This application requires to install"), '<ul>')
					ph = true;
				end
				cgi.printf( '<li>%s %s</li/>', library, V)
			end
			lib = nil
			collectgarbage()
		end
		if ph then cgi.print( '</ul>' ) end
	end
end




local function geti18n()
	local set = {}

	local file = io.open( map( 'html', 'manager.cgi' ) )
	local text = file:read( '*a' )
	file:close()

	for _, key in text:gmatch( 'i18n%s*[(]%s*([\"\'])(.-)%1' ) do 
		set[ key ] = key
	end

	cgi.header( 'Content-Disposition', 'attachment; filename=i18n.txt' );

	for row in io.lines( map( 'html', 'i18n.txt' ) ) do
		local line = row:gsub( '\r', '' )
		local key = line:match( '%["([^"]*)"%]' )
		if key then 
			if set[key] then
				set[key] = nil
			else
				line = line .. "-\45 unused !"
			end
		else
			if line:find( '^}' ) then 

				local sorted = {}
				for key in pairs( set ) do table.insert( sorted, key ) end

				cgi.printf( "--\r\n-- new texts\r\n--\r\n" )
				table.sort( sorted )
				for _, key in ipairs( sorted ) do
					cgi.printf( '["%s"]=\r\n[[%s]];\r\n\r\n' , key, key:gsub( '\\n', '\r\n' ) )
				end
			end
		end
		cgi.printf( '%s\r\n', line )
	end
	return true
end




local authorised = false





local function auth()
	local socket = require('socket')
	local tls = false
	local port = require( 'xmlpro.core' ).port
	if port == 0 then

		tls, port = cgi.socket:getsockname()
	end

	local text = '<get><mb mb=\"9002\"><k k=\"2\">[1-2]</k></mb></get>'
	local request = 'POST /dbms.xml HTTP/1.1\r\nAuthorization: %s\r\nContent-Length: %d\r\n\r\n%s' 
	local s = socket.connect( '127.0.0.1', port )
	if tls then

		local ok, err
		s ,err = require( 'ssl' ).wrap( s, { protocol='tlsv1_2', mode='client', verify='none' } )
		if not s then error(err) end
		ok, err = s:dohandshake()
		if not ok then return nil, err end
	end

	s:send( request:format( cgi.AUTHORIZATION or '', #text, text ) )

	local response = ''
	local status = s:receive():match(" (%d%d%d)")
	if "200" == status then
		local len=0

		repeat 
			local line = s:receive()
			len = line:lower():match( "^content%-length:%s*(%d+)") or len
		until line == ""

		response = s:receive( tonumber(len) or 0 )
	end
	s:close()
	return response, status
end

if not authorised then

	local x = auth()
	if string.find( x, 'i=' ) then
		authorised = true
	elseif cgi.AUTHORIZATION then

		require( 'socket' ).sleep( 3 )
	end
end



if not authorised then
	cgi.status = "401 Unauthorized"
	cgi.header( 'WWW-Authenticate', 'Basic realm="NEC Application Manager"' )
	cgi.print( "Unauthorised!" )
	return
end



findRunningApps()





local cmd, value = '', ''

if cgi.REQUEST_METHOD == 'POST' then
	local name, posted, err
	local length = tonumber( cgi.CONTENT_LENGTH ) or 0

	if length > maxUploadSize then 
		MsgBox = i18n("Package is too large")..'\\n'


		while length>8192 do
			cgi.receive( 8192 )
			length = length - 8192
		end
		cgi.receive( length )
		length = 0
	end

	if length > 0 then
		posted = cgi.receive( length )
		local enctype, boundary = string.match( (cgi.CONTENT_TYPE or ''), '%s*([^;]+);%s*boundary=(.*)$' )
		if enctype == 'multipart/form-data' then
			boundary = string.gsub( boundary, '[][().%+-*?^$]', '%%%1' )
			posted = string.match( posted, boundary..'.-\r\n\r\n(.-)\r\n%-%-'..boundary )
			name, err = preInstall( posted )
			if name then 
				value = name
			else
				MsgBox = i18n("Package upload failed")..'\\n'..err
			end
		else
			posted = unescape( posted )
			cmd, value = string.match( posted, 'cmd=([^=]+)=(.+)' )
		end
	end
elseif cgi.QUERY_STRING > '' then
	cmd, value = string.match( cgi.QUERY_STRING, '&?([^=]+)=([^&]+)' )
end


local has3500 = os.license( 3500 ) > 0




local commands = {

	run = function( value )
		if not runningApps[ value ] then 
			local name 
			if exists( value, value .. '.lua' ) then
				name = '.lua'
			elseif exists( value, value..'-lua' ) then
				name = '-lua'
			else 
				return
			end
			name = map( value, value..name )

			local i = os.execute( name )
			runningApps[ value ] = i
			runningApps[ i ] = value
		end
		return true
	end ,


	stop = function( value )
		require 'permissions'
		local i = runningApps[ value ]
		if i then
			os.kill( i )

			findRunningApps()
			runningApps[ i ]     = nil -- it will be stopped soon
			runningApps[ value ] = nil -- it will be stopped soon
		end
	end ,


	remove = unInstall ,


	cancel = dontInstall ,


	install = doInstall ,


	lock = function( value ) 
		os.rename( map( value, value..'.lua' ) , map( value, value..'-lua' ) ) 
		return true 
	end ,


	unlock = function( value )
		os.rename( map( value, value..'-lua' ) , map( value, value..'.lua' ) ) 
		return true 
	end ,


	no3500 = function() 
		has3500 = false 
		print( "Developer License 3500 ignored" ) 
	end ,


	geti18n = geti18n ,
}


local f = commands[ cmd ]

if f and f( value ) then 

	cgi.header( 'Content-Type', 'text/plain; charset="UTF-8"' )

	return
end





local apps = {}
for Name in lfsdir( map('*') ) do
	local name = lower( Name )
	local app = getAppDetails( name )
	if app then
		table.insert( apps, app )
		apps[ app.name ] = app
	end
end




local selected
if #apps > 0 then
	if value == '' or not apps[ value ] then 
		selected = apps[1].name 
	else
		selected = value
	end
end





cgi.header( 'Content-Type', 'text/html;charset=utf-8' )

cgi.header( 'X-UA-Compatible', 'IE=edge' ) 



cgi.printf( '<!DOCTYPE HTML>\r\n\z
	<html>\r\n\z
	<head>\r\n\z
	<meta charset="UTF-8" />\r\n\z
	<title>%s</title>\r\n\z
	<link rel="SHORTCUT ICON" href="/html/apps/favicon.ico">\r\n\z
	<link rel="stylesheet" type="text/css" href="common.css" >\r\n\z
	<link rel="stylesheet" type="text/css" href="manager.css" >\r\n\z
	<script>var MsgBox="%s";\r\n\z
	var confirm1="%s";\r\n\z
	var confirm2="%s";\r\n', 
	i18n("NEC PBX Application Manager"),
	MsgBox or '',
	i18n("Do you really want to remove this application\nand all its files and directories ?"):gsub('\n', '\\n'),
	i18n("Are you sure ?")
)

cgi.printf( 'var MemoryReason = "%s";\r\n',		i18n("there is not enough memory available") )
cgi.printf( 'var LuaVMReason = "%s";\r\n',		i18n("we have not not enough Lua VMs") )
cgi.printf( 'var ListenReason = "%s";\r\n',		i18n("a required TCP port is already in use") )
cgi.printf( 'var ConnectReason =" %s";\r\n',	i18n("a not sharable TCP connection is being used") )
cgi.printf( 'var UDPReason = "%s";\r\n',	 	i18n("a required UDP port is being used") )


cgi.print( 'var AppNames=[ ' )
local comma = ''
for _, app in ipairs( apps ) do
	cgi.printf( '%s"%s"', comma, app.name )
	comma = ','
end
cgi.print( ' ];' )


local function listR( R, name )
	if not R[ name ] then return end

	local comma=''
	cgi.printf( '%s:[', name )
	for _,port in pairs( R[name] ) do
		cgi.printf( '%s"%s"', comma, port )
		comma = ','
	end
	cgi.printf( '],' )
end



cgi.print( '\r\nvar Resources={\r\n' )
comma = ''
for _,app in ipairs( apps ) do
	local R = app.M or app.N
	R = R and R.resources
	if R then
		cgi.printf( '%s\t"%s":{', comma, app.name )
		comma = ',\r\n'
		if R.memory then
			cgi.printf( 'memory:%d,', R.memory )
		end
		listR( R, 'listen' )
		listR( R, 'connect' )
		listR( R, 'udp' )

		cgi.printf('LuaVMs:%d }', R.LuaVMs or 1 )
	end
end
cgi.print
(
	' \r\n};\r\n</script>\r\n\z
	<script src="manager.js" type="text/javascript" ></script>\r\n\z
	</head>\r\n'
)




cgi.printf( 
	'<body>\r\n\z
	<h1>%s <small>%s</small></h1>\r\n\z
	<h2>\r\n\z
	<form name="upload" action="manager.cgi" method="post" enctype="multipart/form-data">\r\n\z
	<a class="button"><div class="upload">\r\n\z
	<input type="file" id="uploadfile" name="uploadfile"/></div>\r\n\z
	%s</a>\r\n\z
	</form>\r\n\z
	<a class="button hide" id="refresh" href="javascript:doPoll();"><img src="refresh.png" />%s</a>\r\n\z
	%s</h2>\r\n\z
	<div class="apps">\r\n\z
	<form name="post" action="manager.cgi" method="post"><input type="hidden" name="cmd" /></form>\r\n\z
	<ul class="apps" id="applist">\r\n' ,
	i18n("Application Manager"), '1.8.3', i18n("Install"), i18n("Refresh"), i18n("Installed Applications"))




for _,app in ipairs( apps ) do
	local M = app.N or app.M
	local intl = M.description
	local natl = intl[ lang ] or {}

	cgi.printf
	(
		'<li style="background-image:url(/html/apps%s/icon.png);" onclick="details(\'%s\')" id="list-%s" class="%s"><h3>%s&nbsp;<small>%s</small></h3>%s</li>\r\n',
		app.icon and '/'..app.name or '',
		app.name,
		app.name,
		selected == app.name and 'sel' or '',
		natl.name or intl.name or i18n("Applet"),
		M.version or '',
		natl.summary or intl.summary or ''
	)
end

cgi.print( '</ul></div>\r\n<div class="details" id="appdetails">\r\n')




for _,app in ipairs(apps) do
	local M = app.N or app.M
	local intl = M.description
	local natl = intl[ lang ] or {}
	local licenses 

	if not app.license or app.license == 0 or has3500 then
		licenses = 1 -- always on
	else
		licenses = os.license( app.license )
		if licenses < 1 then
			app.canRun = false
			app.config = false
		end
	end

	cgi.printf
	(
		'<div id="detail-%s" class="%s" data-state="5"><h4 class="buttons">%s ',
		app.name,
		selected == app.name and 'sel' or '',
		natl.name or intl.name or i18n("Details")
	)

	if app.name ~= 'html' and app.name ~= 'lib' then


		if app.config then



			local cfgrun = M.RCPORT and ' state3' or ''
			cgi.printf
			(
				'<a class="button%s" id="%s-config" href="/html/apps/%s/config.cgi" target="_blank"><img src="config.png" />%s</a>',
				cfgrun, 
				app.name, 
				app.name, 
				i18n("Configure")
			)
		end

		if app.canRun then

			cgi.printf
			(
				'<a class="button state0" id="%s-run" href="javascript:doRun(\'%s\')"><img src="play.png" />%s</a>',
				app.name, 
				app.name, 
				i18n("Start")
			)
		end


		cgi.printf
		(
			'<a class="button state3 state4" id="%s-stop" href="javascript:doStop(\'%s\')"><img src="stop.png" />%s</a>',
			app.name, 
			app.name, 
			i18n("Stop")
		)


		if app.M then

			cgi.printf
			(
				'<a class="button state0" id="%s-remove" href="javascript:doRemove(\'%s\')"><img src="trash.png" />%s</a>',
				app.name, 
				app.name, 
				i18n("Remove")
			)


			if app.canRun and licenses > 0 then
				cgi.printf
				(
					'<a class="button%s" id="%s-unlock" href="javascript:doUnlock(\'%s\')"><img src="unlock.png" />%s</a>',
					app.isLocked and "" or " hide", 
					app.name, 
					app.name, 
					i18n("Enable")
				)
				cgi.printf
				(
					'<a class="button%s" id="%s-lock" href="javascript:doLock(\'%s\')"><img src="lock.png" />%s</a>',
					app.isLocked and " hide" or "", 
					app.name, 
					app.name,
					i18n("Disable")
				)
			end


			for _, button in ipairs( intl.buttons or {} ) do
				if button.img and button.href then
					cgi.printf( '<a class="button %s" href="%s"', button.class or "", button.href )
					if button.target then
						cgi.printf( ' target="%s"', button.target)
					end
					if button.id then
						cgi.printf( ' id="%s-%s"', app.name, button.id)
					end
					cgi.printf( '><img src="%s" />%s</a>', button.img, i18n( button[lang] or button[1] or "" ) )
				end 
			end
		end
	end
	cgi.print( '</h4>' )



	cgi.printf
	(	
		'<div class="reason">%s %s<span id="%s-reason"></span><br />%s</div>',

		natl.name or intl.name,
		i18n("can't be started because "),
		app.name,
		i18n("You may have to stop another application first")
	)

	paragraph( i18n("Summary:"), '<br />%s', natl.summary or intl.summary)
	paragraph( i18n("Description:"), '<br />%s', (natl.detailed or intl.detailed):gsub('<script>.-</script>', '') )
	paragraph( i18n("Home Page:"), '<a href="%s">%s</a>', intl.homepage, intl.homepage)
	paragraph( i18n("Maintained by:"), '%s', intl.maintainer)
	paragraph( i18n("Update URL:"), '<a href="%s">%s</a>', intl.updateURL, intl.updateURL)
	if app.license then
		paragraph( i18n("Required License Code:"), '%s', (app.license==0) and i18n("free") or tostring(app.license) )
		if licenses <= 0 then
			cgi.printf( '<em>%s<br />%s</em><br />',	i18n("This license is not installed."), i18n("It is required to start or configure the application"))
		end
	end
	paragraph( i18n("Copyright:"), '%s', intl.copyright)


	if app.N then

		cgi.printf
		(
			'<h4 class="buttons hr">%s\r\n\z
			<a class="button" id="%s-install" href="javascript:doInstall(\'%s\')"><img src="ok.png" />%s</a>\r\n\z
			<a class="button" id="%s-cancel" href="javascript:doCancel( \'%s\')"><img src="cancel.png" />%s </a></h4>\r\n',

			i18n("Uploaded Package ready for Installation"),
			app.name,	app.name, i18n("Install"),
			app.name,	app.name, i18n("Cancel")
		)

		if app.N.permissions then listPermissions( app.N.permissions ) end
		if app.N.requirements then listRequirements( app.N.requirements ) end


		if app.N.version then paragraph( i18n("New Version:"), "%s", app.N.version) end
	end


	if app.M then
		paragraph( i18n("Installed Version:"), '%s', app.M.version or '')
	end
	cgi.print( '</div>\r\n' )
end
cgi.print( '</div></body></html>' )

















